<?php
/**
 * Plugin Name: Cario Shipping Plugin
 * Description: Cario Shipping Plugin for WooCommerce
 * Version: 1.0.0
 * Text Domain: cario
 */

require 'bootstrap.php';

if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {

    function cario_shipping_plugin() {

        if (!class_exists('Cario_Shipping_Plugin')) {

            class Cario_Shipping_Plugin extends WC_Shipping_Method
            {
                public $cario_access_token;

                public function __construct() 
                {
                    $this->id = 'cario'; 

                    $this->method_title = __('Cario Shipping Plugin', 'cario');  

                    $this->method_description = __('Cario shipping plugin for Woocommerce', 'cario'); 

                    $this->init();

                    $this->enabled = isset($this->settings['enabled']) ? $this->settings['enabled'] : 'yes';

                    $this->tax_status = 'non-taxable';

                    $this->cario_access_token = isset($this->settings['cario_access_token']) ? $this->settings['cario_access_token'] : null;
                }
 
                function init() 
                {
                    $this->init_form_fields(); 

                    $this->init_settings(); 
 
                    add_action( 'woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
                }
 
                function init_form_fields() 
                {
                    $this->form_fields = [
                        'enabled' => [
                            'title' => __('Enable', 'cario'),
                            'type' => 'checkbox',
                            'description' => __( 'Enable Cario shipping plugin.', 'cario'),
                            'default' => 'yes',
                        ],
                        'cario_access_token' => [
                            'title' => __('Cario Access Token', 'cario'),
                            'type' => 'textarea',
                            'description' => __( 'Your access token to Cario API.', 'cario'),
                            'default' => null,
                        ],
                    ];
                }
 
                public function calculate_shipping($package = []) 
                {
                    $country = $package['destination']['country'] ?? null;
                    $city = $package['destination']['city'] ?? null;
                    
                    if(!($country && $city)) {
                        return;
                    }

                    $transport_units = [];

                    foreach($package['contents'] as $product) {
                        $attributes = get_post_meta($product['product_id'], '_product_attributes', true);

                        $transport_units[] = [
                            'containerType' => $attributes['containertype']['value'],
                            'quantity' => $product['quantity'],
                            'length' => get_post_meta($product['product_id'], '_length', true),
                            'width' => get_post_meta($product['product_id'], '_width', true),
                            'height' => get_post_meta($product['product_id'], '_height', true),
                            'weight' => get_post_meta($product['product_id'], '_weight', true),
                        ];
                    }

                    list($country, $state) = explode(':', get_option('woocommerce_default_country'));

                    $response = wp_remote_post(CARIO_PLUGIN_BASE_URL . '/api/woocommerce/quote', [
                        'method' => 'POST',
                        'timeout' => 180,
                        'headers' => [
                            'X-Cario-Plugin-Key' => CARIO_PLUGIN_APP_KEY,
                            'X-Cario-Access-Token' => $this->cario_access_token,
                            'Content-Type' => 'application/json',
                            'Accept' => 'application/json',
                        ],
                        'body' => json_encode([
                            'origin' => [
                                'country' => $country,
                                'state' => $state,
                                'postcode' => get_option('woocommerce_store_postcode'),
                                'city' => get_option('woocommerce_store_city'),
                                'address' => get_option('woocommerce_store_address'),
                                'address_2' => get_option('woocommerce_store_address_2'),   
                            ],
                            'destination' => $package['destination'],
                            'transport_units' => $transport_units,
                        ]),
                    ]);

                    if(is_wp_error($response)) {
                         wc_add_notice('Unable to find carrier service for your order. Please try again.', 'error');
                    }
                    else {
                        if ($response['response']['code'] != 200) {
                             wc_add_notice('Unable to find carrier service for your order. Please try again.', 'error');
                        }
                        else {
                            $quotes = json_decode($response['body'], true);

                            foreach($quotes as $quote) {
                                $this->add_rate([
                                    'id' => $quote['id'],
                                    'label' => $quote['label'],
                                    'cost' => $quote['cost'],
                                    'taxes' => false,
                                    'meta_data' => [
                                        'service_code' => $quote['id'],
                                    ],
                                ]);
                            }
                        }
                    }
                }
            }
        }
    }
 
    add_action('woocommerce_shipping_init', 'cario_shipping_plugin');
 
    function add_cario_shipping_plugin($methods) 
    {
        $methods[] = 'Cario_Shipping_Plugin';

        return $methods;
    }
 
    add_filter('woocommerce_shipping_methods', 'add_cario_shipping_plugin');


    function add_create_consignment_action($actions) 
    {
        global $theorder;

        if (!$theorder->is_paid() || !$theorder->has_shipping_method('cario') || get_post_meta($theorder->get_order_number(), '_wc_order_consignment_created', true)) {
            return $actions;
        }

        $actions['wc_custom_order_action'] = __('Create Consignment', 'cario');

        return $actions;
    }

    add_action('woocommerce_order_actions', 'add_create_consignment_action');


    function process_create_consignment_action($order) 
    {
        $order_number = trim(str_replace('#', '', $order->get_order_number()));
        $carrier_service = [];

        foreach($order->get_shipping_methods() as $shipping_method) {
            $service_code = null;

            foreach($shipping_method->get_meta_data() as $meta_data) {
                if($meta_data->key == 'service_code') {
                    $service_code = $meta_data->value;
                    break;
                }
            }

            $carrier_service = [
                'code' => $service_code,
                'title' => $shipping_method->get_data()['name'],
                'price' => $shipping_method->get_data()['total'],
            ];

            break;
        }

        list($country, $state) = explode(':', get_option('woocommerce_default_country'));

        $pickup_address = [
            'name' => get_option('blogname'),
            'country' => $country,
            'state' => $state,
            'postcode' => get_option('woocommerce_store_postcode'),
            'city' => get_option('woocommerce_store_city'),
            'address' => get_option('woocommerce_store_address'),
            'address_2' => get_option('woocommerce_store_address_2'),   
        ];

        $delivery_address = $order->get_data()['shipping'];

        $transport_units = [];

        foreach($order->get_items() as $product) {
            $attributes = get_post_meta($product['product_id'], '_product_attributes', true);

            $transport_units[] = [
                'containerType' => $attributes['containertype']['value'],
                'quantity' => $product['quantity'],
                'length' => get_post_meta($product['product_id'], '_length', true),
                'width' => get_post_meta($product['product_id'], '_width', true),
                'height' => get_post_meta($product['product_id'], '_height', true),
                'weight' => get_post_meta($product['product_id'], '_weight', true),
            ];
        }

        $cario_shipping_plugin = new Cario_Shipping_Plugin();

        $response = wp_remote_post(CARIO_PLUGIN_BASE_URL . '/api/woocommerce/consignment', [
            'method' => 'POST',
            'timeout' => 180,
            'headers' => [
                'X-Cario-Plugin-Key' => CARIO_PLUGIN_APP_KEY,
                'X-Cario-Access-Token' => $cario_shipping_plugin->cario_access_token,
                'X-WooCommerce-Shop' => get_site_url(),
                'Content-Type' => 'application/json',
                'Accept' => 'application/json',
            ],
            'body' => json_encode([
                'order_number' => $order_number,
                'carrier_service' => $carrier_service,
                'pickup_address' => $pickup_address,
                'delivery_address' => $delivery_address,
                'transport_units' => $transport_units,
            ]),
        ]);

        if(is_wp_error($response)) {
            update_post_meta($order->get_order_number(), '_wc_order_consignment_error', 'yes');
        }
        else {
            if ($response['response']['code'] != 200) {
                update_post_meta($order->get_order_number(), '_wc_order_consignment_error', 'yes');
            }
            else {
                $consignment = json_decode($response['body'], true);

                $remote_label_url = CARIO_PLUGIN_BASE_URL . '/storage/labels/' . $consignment['consignment_id'] . '.pdf';
                $local_label_url = plugin_dir_url(__FILE__) . 'labels/' . $consignment['consignment_id'] . '.pdf';
                $local_label_dir = plugin_dir_path(__FILE__) . 'labels/';

                if(!file_exists($local_label_dir)) {
                    mkdir($local_label_dir);
                }

                $response = wp_remote_get($remote_label_url, [ 
                    'timeout'  => 180,
                    'stream'   => true,
                    'filename' => $local_label_dir . $consignment['consignment_id'] . '.pdf',
                ]);

                $consignment_link = '<a class="printjs" href="' . $local_label_url .'">' . $consignment['connote_number'] . '</a>';

                $order->add_order_note(sprintf(__('Consignment created with Connote # </b>%s</b>.', 'cario'), $consignment_link));
                
                update_post_meta($order->get_order_number(), '_wc_order_consignment_created', 'yes');
                update_post_meta($order->get_order_number(), '_wc_order_consignment_printed', 'no');
            }
        }
    }

    add_action('woocommerce_order_action_wc_custom_order_action', 'process_create_consignment_action');

    function enqueue_print_js() 
    {   
        wp_enqueue_script('cario', plugin_dir_url(__FILE__) . '/printjs/print.min.js', [], false, true);
    }

    add_action('admin_enqueue_scripts', 'enqueue_print_js');

    function footer_print_js() 
    {
        ?>
            <script>
                jQuery(document).ready(function() {
                    jQuery('.printjs').click(function(e) {
                        e.preventDefault();
                        printJS(jQuery(this).attr('href'));
                    });
                });
            </script>
        <?php

        if(get_post_meta(get_the_ID(), '_wc_order_consignment_printed', true) == 'no') {
        ?>
            <script>
                jQuery(document).ready(function() {
                    printJS(jQuery('.printjs').first().attr('href'));
                });
            </script>
        <?php
            update_post_meta(get_the_ID(), '_wc_order_consignment_printed', 'yes'); 
        }

        if(get_post_meta(get_the_ID(), '_wc_order_consignment_error', true) == 'yes') {
        ?>
            <script>
                jQuery(document).ready(function() {
                    jQuery('#message').hide();
                });
            </script>
        <?php
            update_post_meta(get_the_ID(), '_wc_order_consignment_error', 'no'); 
        }
    }

    add_action('admin_footer', 'footer_print_js');

    function create_consignment_error () 
    {
        if(get_post_meta(get_the_ID(), '_wc_order_consignment_error', true) == 'yes') {
            ?>
                <div class="notice notice-error is-dismissible">
                    <p><?php _e( 'Unable to create consignment. Please try again.', 'cario' ); ?></p>
                </div>
            <?php
        }
    }

    add_action('admin_notices', 'create_consignment_error');

}
